/**
 * Copyright (c) 2005, David Holm
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the project nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __OF_H
#define __OF_H

#include <stdarg.h>

typedef void *ihandle;
typedef void *phandle;
typedef int (*prom_entry)(void *);

typedef struct mem_region_s {
	unsigned long address;
	unsigned int  len;
} mem_region_t;

typedef struct mem_translation_s {
	unsigned long vaddr;
	unsigned int  len;
	unsigned long paddr;
	unsigned int  mode;
} mem_translation_t;

typedef struct of_s {
	prom_entry	entry;

	ihandle		stdin;
	ihandle		stdout;
} of_t;

typedef struct of_args {
		const char *service;
		int         nargs;
		int         nret;
		void       *args[10];
	} of_targs;

void         of_init(prom_entry entry);
phandle      of_find_device(const char *name);
int          of_get_property(phandle package, const char *name, void *mem, unsigned int len);
int          of_exit();

void         of_puts(char *s);
void         of_printf(char *fmt, ...);

void        *of_claim(void *vaddr, unsigned int len, unsigned int align);
void         of_release(void *vaddr, unsigned int len);

#endif /* __OF_H */
