/**
 * Copyright (c) 2005, David Holm
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the project nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "string.h"

void *memcpy(void *dst, void *src, unsigned int len)
{
	unsigned char *d = (unsigned char *) dst;
	unsigned char *s = (unsigned char *) src;

	while (len--) {
		*d++ = *s++;
	}

	return dst;
}

void flush_icache(void *s, unsigned int len)
{
	unsigned int   i;
	unsigned char *p = (unsigned char *) s;

	for (i = 0; i < len; i += 8, p += 8) {
		asm("dcbst 0, %0" : : "r" (p) : "memory");
	}
	asm("sync");

	p = (unsigned char *) s;
	for (i = 0; i < len; i += 8, p += 8) {
		asm("icbi 0, %0; sync" : : "r" (p) : "memory");
	}
	asm("sync");
	asm("isync");
}

char *strcat(char *s1, const char *s2)
{
	char *t = s1;

	if (!s1 || !s2) {
		return 0L;
	}

	for (; *t; ++t) {
		continue;
	}

	while ((*t++ = *s2++) != '\0') {
		continue;
	}

	return s1;
}

int strlen(const char *str)
{
	const char  *s;
	unsigned int i;

	if (!str) {
		return 0;
	}

	for (s = str, i = 0; *s; ++s, ++i) {
		continue;
	}

	return i;
}

int strnlen(const char *str, unsigned int len)
{
	const char  *s;
	unsigned int i;

	if (!str) {
		return 0;
	}

	for (s = str, i = 0; *s && i < len; ++s, ++i) {
		continue;
	}

	return i;
}

int strcmp(const char *s1, const char *s2)
{
	if (!s1 || !s2) {
		/* FIXME: Assert */
		return -1;
	}

	while (*s1 == *s2++) {
		if (*s1++ == 0) {
			return 0;
		}
	}

	return (*(const unsigned char *) s1 - *(const unsigned char *) --s2);
}

void *memset(void *s, int c, unsigned int len)
{
	char *p = (char *) s;

	while (len--) {
		*p++ = c;
	}

	return s;
}
