/**
 * Copyright (c) 2005, David Holm
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the project nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __STRING_H
#define __STRING_H

#define NULL 0L

/**
 * Fill memory with constant
 * \param s Pointer to memory
 * \param c Constant
 * \param n Bytes to fill
 * \returns Pointer to memory
 */
void *memset(void *s, int c, unsigned int n);

/**
 * Copy memory area
 * \param dst Pointer to destination
 * \param src Pointer to source
 * \param len Length of block to copy
 * \returns Pointer to destination
 */
void *memcpy(void *dst, void *src, unsigned int len);

/**
 * Flush the instruction cache of the specified memory region
 * \param s   Base address of memory region
 * \param len Length of memory region
 */
void  flush_icache(void *s, unsigned int len);

/**
 * Concatenate two strings in the first one's address space
 * \param s1 String 1
 * \param s2 String 2
 * \returns Pointer to resulting string
 */
char *strcat(char *s1, const char *s2);

/**
 * Calculate length of string
 * \param str String
 * \param len Maximum length of str
 * \returns Length of string in characters
 */
int   strnlen(const char *str, unsigned int len);

/**
 * Calculate length of null terminated string
 * \param str String
 * \returns Length of string in characters
 */
int strlen(const char *str);

/**
 * Compare two strings
 * \param s1 String 1
 * \param s2 String 2
 * \returns Smaller than, equal to or greater than zero if s1 is smaller, equal or greater than s2
 */
int   strcmp(const char *s1, const char *s2);

#endif /* __STRING_H */
